(* type t *)
type 'a t = Empty | Node of ('a t * 'a * 'a t)
(* size : 'a t -> int *)
let rec size = function Empty       -> 0
                      | Node(l,_,r) -> size l + size r + 1
(* height : 'a t -> int *)
let rec height = function
 | Empty       -> 0
 | Node(l,_,r) -> max (height l) (height r) + 1
(* of_list : 'a list -> 'a t *)
let rec of_list = function
 | []    -> Empty
 | x::xs -> Node(Empty,x,of_list xs)
(* make : 'a list -> 'a t *)
let rec make = function
 | [] -> Empty
 | xs ->
  let m = Lst.length xs / 2 in
  let (ys,zs) = Lst.split_at m xs in
  Node (make ys,Lst.hd zs,make(Lst.tl zs))
(* insert : ('a -> 'a -> int) -> 'a -> 'a t -> 'a t *)
let rec insert c v = function
 | Empty       -> Node(Empty,v,Empty)
 | Node(l,w,r) -> if c v w < 1 then Node(insert c v l,w,r)
                               else Node(l,w,insert c v r)
(* search_tree ('a -> 'a -> int) -> 'a list -> 'a t *)
let search_tree c =
 Lst.foldl (fun x y -> insert c y x) Empty
(* flatten : 'a t -> 'a list *)
let rec flatten = function
 | Empty       -> []
 | Node(l,x,r) -> (flatten l)@(x::flatten r)
(*  *)
